window.Gleam.initSearch([{"doc":"outil","title":"outil","content":"# outil\n\n[![Package Version](https://img.shields.io/hexpm/v/outil)](https://hex.pm/packages/outil)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/outil/)\n\nA library for writing command line tools. Like so:\n\n```gleam\nimport gleam/erlang\nimport gleam/io\nimport gleam/list\nimport gleam/result\nimport gleam/string\nimport outil.{command, print_usage_and_exit}\nimport outil/arg\nimport outil/opt\n\nfn say_hello(args) {\n  use cmd &lt;- command(&quot;hello&quot;, &quot;Say hello to someone&quot;, args)\n  use name, cmd &lt;- arg.string(cmd, &quot;name&quot;)\n  use enthusiasm, cmd &lt;- opt.int(cmd, &quot;enthusiasm&quot;, &quot;How enthusiastic?&quot;, 1)\n\n  try name = name(cmd)\n  try enthusiasm = enthusiasm(cmd)\n\n  let message = &quot;Hello, &quot; &lt;&gt; name &lt;&gt; string.repeat(&quot;!&quot;, enthusiasm)\n\n  Ok(io.println(message))\n}\n\npub fn main() {\n  // Erlang is not required, this example just uses it for getting ARGV\n  let args = erlang.start_arguments()\n\n  say_hello(args)\n  |&gt; result.map_error(print_usage_and_exit)\n}\n```\n\nIf you don&#39;t fancy this style of programming, check out [glint] or [Awesome Gleam] for alternatives.\n\nOutil is not going to have many cool features for building comprehensive command line\ninterfaces. It is meant to fit simple programs with simple needs.\n\n[glint]: https://github.com/tanklesxl/glint\n[Awesome Gleam]: https://github.com/gleam-lang/awesome-gleam#cli\n\n## Quick start\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\nOutil is available on Hex and can be added to your Gleam project like so:\n\n```sh\ngleam add outil\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/outil&gt;.\n\n## Changelog\n\n### 0.3.2\n\n* Fixed a bug where the automatic `--help` flag didn&#39;t react unless the command had positional arguments.\n\n### 0.3.1\n\n* Added convenience helpers print_usage and print_usage_and_exit for handling command errors.\n\n### 0.3.0\n\n* Expanded the return type of commands to include a way for the command code itself to return errors.\n* **BREAKING** Some types were renamed to support the above, and be more clear.\n\n### 0.2.1\n\n* Fixed some docs.\n\n### 0.2.0\n\n* **BREAKING** Simplified command implementation. The function is the command now, it no longer returns a `Command` value.\n\n### 0.1.0\n\n* Hello world!\n* Something kind of working.\n","url":"index.html"},{"doc":"outil/error","title":"Reason","content":"pub type Reason {\n  MalformedArgument(String, String)\n  MissingArgument(String)\n  OutOfPlaceOption(String)\n}\n Errors that can occur when parsing command line arguments.\nMalformedArgument(String, String)\n\n\nMissingArgument(String)\n\n\nOutOfPlaceOption(String)\n\n\nSynonyms:\noutil/error.Reason\noutil/error Reason","url":"outil/error.html#Reason"},{"doc":"outil/error","title":"outil/error","content":"","url":"outil/error.html"},{"doc":"outil","title":"Argument","content":"pub type Argument {\n  BoolArgument(name: String)\n  FloatArgument(name: String)\n  IntArgument(name: String)\n  StringArgument(name: String)\n}\n A positional command line argument.\nBoolArgument(name: String)\n\n\nFloatArgument(name: String)\n\n\nIntArgument(name: String)\n\n\nStringArgument(name: String)\n\n\nSynonyms:\noutil.Argument\noutil Argument","url":"outil.html#Argument"},{"doc":"outil","title":"Command","content":"pub type Command {\n  Command(\n    name: String,\n    description: String,\n    arguments: List(Argument),\n    options: List(Opt),\n    argv: List(String),\n  )\n}\n A command line interface to a run function.\nCommand(\n  name: String,\n  description: String,\n  arguments: List(Argument),\n  options: List(Opt),\n  argv: List(String),\n)\n\n\nSynonyms:\noutil.Command\noutil Command","url":"outil.html#Command"},{"doc":"outil","title":"CommandResult","content":"pub type CommandResult(a, b) =\n  Result(a, CommandReturn(b))\n The result of executing a command.\n\nSynonyms:\noutil.CommandResult\noutil CommandResult","url":"outil.html#CommandResult"},{"doc":"outil","title":"CommandReturn","content":"pub type CommandReturn(a) {\n  CommandLineError(reason: Reason, usage: String)\n  CommandError(a)\n  Help(usage: String)\n}\n Non-normal return values from executing a command.\nCommandLineError(reason: Reason, usage: String)\n An error in parsing the command line.\n\nCommandError(a)\n An error in executing the command, `a` is your error value.\n\nHelp(usage: String)\n The user asked for help.\n\nSynonyms:\noutil.CommandReturn\noutil CommandReturn","url":"outil.html#CommandReturn"},{"doc":"outil","title":"Configure","content":"pub type Configure(a, b, c) =\n  fn(RunCommand(a, c), Command) -&gt; b\n The type of continuation functions in building a command.\n\nSynonyms:\noutil.Configure\noutil Configure","url":"outil.html#Configure"},{"doc":"outil","title":"Opt","content":"pub type Opt {\n  Opt(\n    long: String,\n    short: Option(String),\n    description: String,\n    value: OptValue,\n  )\n}\n A command line option/flag.\nOpt(\n  long: String,\n  short: Option(String),\n  description: String,\n  value: OptValue,\n)\n\n\nSynonyms:\noutil.Opt\noutil Opt","url":"outil.html#Opt"},{"doc":"outil","title":"OptValue","content":"pub type OptValue {\n  BoolOpt\n  FloatOpt(default: Float)\n  IntOpt(default: Int)\n  StringOpt(default: String)\n}\n The type and default value of an option.\nBoolOpt\n\n\nFloatOpt(default: Float)\n\n\nIntOpt(default: Int)\n\n\nStringOpt(default: String)\n\n\nSynonyms:\noutil.OptValue\noutil OptValue","url":"outil.html#OptValue"},{"doc":"outil","title":"RunCommand","content":"pub type RunCommand(a, b) =\n  fn(Command) -&gt; CommandResult(a, b)\n The type of functions that execute a command.\n\nSynonyms:\noutil.RunCommand\noutil RunCommand","url":"outil.html#RunCommand"},{"doc":"outil","title":"command","content":"pub fn command(name: String, description: String, argv: List(\n    String,\n  ), continue: fn(Command) -&gt; a) -&gt; a\n Create a command with the given name and description, and pass it to the\n given continuation function for further configuration.\n\n The command gets a default implementation that returns an error.\nSynonyms:\noutil.command\noutil command","url":"outil.html#command"},{"doc":"outil","title":"parse_bool","content":"pub fn parse_bool(arg: String) -&gt; Result(Bool, Nil)\n Parse a Bool from a string.\nSynonyms:\noutil.parse_bool\noutil parse_bool","url":"outil.html#parse_bool"},{"doc":"outil","title":"print_usage","content":"pub fn print_usage(return: CommandReturn(a)) -&gt; Nil\n Convenience function for handling the result of executing a command.\n\n Print usage if there was a command line error or if the user asked for it.\n\n Does nothing if the command returned an application error, assuming that\n the application will handle it.\nSynonyms:\noutil.print_usage\noutil print_usage","url":"outil.html#print_usage"},{"doc":"outil","title":"print_usage_and_exit","content":"pub fn print_usage_and_exit(return: CommandReturn(a)) -&gt; Nil\n Convenience function for handling the result of executing a command.\n\n Print usage if there was a command line error or if the user asked for it.\n\n Exit with 2 if there was a command line error, 1 if there was a command\n error, or 0 if the command was successful.\nSynonyms:\noutil.print_usage_and_exit\noutil print_usage_and_exit","url":"outil.html#print_usage_and_exit"},{"doc":"outil","title":"outil","content":"","url":"outil.html"},{"doc":"outil/help","title":"UseArgs","content":"pub type UseArgs(a) =\n  fn(List(String)) -&gt; Result(a, Reason)\n A function using the command line arguments which can return command error reasons.\n\nSynonyms:\noutil/help.UseArgs\noutil/help UseArgs","url":"outil/help.html#UseArgs"},{"doc":"outil/help","title":"wrap_usage","content":"pub fn wrap_usage(cmd: Command, continue: fn(List(String)) -&gt;\n    Result(a, Reason)) -&gt; Result(a, CommandReturn(b))\n Given a command, if the command line arguments contain `--help` or `-h`\n then return a help &quot;error&quot; with the usage string.\n Otherwise, pass the arguments to the given function and if it returns an\n error, wrap it in a `CommandLineError` with the usage string.\nSynonyms:\noutil/help.wrap_usage\noutil/help wrap_usage","url":"outil/help.html#wrap_usage"},{"doc":"outil/help","title":"outil/help","content":"","url":"outil/help.html"},{"doc":"outil/arg","title":"bool","content":"pub fn bool(cmd: Command, name: String, continue: fn(\n    fn(Command) -&gt; Result(Bool, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a positional bool argument to the command before continuing.\nSynonyms:\noutil/arg.bool\noutil/arg bool","url":"outil/arg.html#bool"},{"doc":"outil/arg","title":"float","content":"pub fn float(cmd: Command, name: String, continue: fn(\n    fn(Command) -&gt; Result(Float, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a positional float argument to the command before continuing.\nSynonyms:\noutil/arg.float\noutil/arg float","url":"outil/arg.html#float"},{"doc":"outil/arg","title":"int","content":"pub fn int(cmd: Command, name: String, continue: fn(\n    fn(Command) -&gt; Result(Int, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a positional int argument to the command before continuing.\nSynonyms:\noutil/arg.int\noutil/arg int","url":"outil/arg.html#int"},{"doc":"outil/arg","title":"string","content":"pub fn string(cmd: Command, name: String, cont: fn(\n    fn(Command) -&gt; Result(String, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a positional string argument to the command before continuing.\nSynonyms:\noutil/arg.string\noutil/arg string","url":"outil/arg.html#string"},{"doc":"outil/arg","title":"outil/arg","content":"","url":"outil/arg.html"},{"doc":"outil/opt","title":"bool","content":"pub fn bool(cmd: Command, long: String, description: String, continue: fn(\n    fn(Command) -&gt; Result(Bool, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a named bool option to the command before continuing.\nSynonyms:\noutil/opt.bool\noutil/opt bool","url":"outil/opt.html#bool"},{"doc":"outil/opt","title":"bool_","content":"pub fn bool_(cmd: Command, long: String, short: Option(String), description: String, continue: fn(\n    fn(Command) -&gt; Result(Bool, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a bool option with a short name to the command before continuing.\nSynonyms:\noutil/opt.bool_\noutil/opt bool_","url":"outil/opt.html#bool_"},{"doc":"outil/opt","title":"float","content":"pub fn float(cmd: Command, long: String, description: String, default: Float, continue: fn(\n    fn(Command) -&gt; Result(Float, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a named float option to the command before continuing.\nSynonyms:\noutil/opt.float\noutil/opt float","url":"outil/opt.html#float"},{"doc":"outil/opt","title":"float_","content":"pub fn float_(cmd: Command, long: String, short: Option(String), description: String, default: Float, continue: fn(\n    fn(Command) -&gt; Result(Float, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a float option with a short name to the command before continuing.\nSynonyms:\noutil/opt.float_\noutil/opt float_","url":"outil/opt.html#float_"},{"doc":"outil/opt","title":"int","content":"pub fn int(cmd: Command, long: String, description: String, default: Int, continue: fn(\n    fn(Command) -&gt; Result(Int, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a named int option to the command before continuing.\nSynonyms:\noutil/opt.int\noutil/opt int","url":"outil/opt.html#int"},{"doc":"outil/opt","title":"int_","content":"pub fn int_(cmd: Command, long: String, short: Option(String), description: String, default: Int, continue: fn(\n    fn(Command) -&gt; Result(Int, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add an int option with a short name to the command before continuing.\nSynonyms:\noutil/opt.int_\noutil/opt int_","url":"outil/opt.html#int_"},{"doc":"outil/opt","title":"named_opt_parser","content":"pub fn named_opt_parser(long: String, short: Option(String), parser: fn(\n    String,\n  ) -&gt; Result(a, Nil), default: a) -&gt; fn(List(String)) -&gt;\n  Result(a, Reason)\n\nSynonyms:\noutil/opt.named_opt_parser\noutil/opt named_opt_parser","url":"outil/opt.html#named_opt_parser"},{"doc":"outil/opt","title":"string","content":"pub fn string(cmd: Command, long: String, description: String, default: String, continue: fn(\n    fn(Command) -&gt; Result(String, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a named string option to the command before continuing.\nSynonyms:\noutil/opt.string\noutil/opt string","url":"outil/opt.html#string"},{"doc":"outil/opt","title":"string_","content":"pub fn string_(cmd: Command, long: String, short: Option(String), description: String, default: String, continue: fn(\n    fn(Command) -&gt; Result(String, CommandReturn(a)),\n    Command,\n  ) -&gt; b) -&gt; b\n Add a string option with a short name to the command before continuing.\nSynonyms:\noutil/opt.string_\noutil/opt string_","url":"outil/opt.html#string_"},{"doc":"outil/opt","title":"with_named_option","content":"pub fn with_named_option(cmd: Command, opt: Opt, default: a, parse: fn(\n    String,\n  ) -&gt; Result(a, Nil), continue: fn(\n    fn(Command) -&gt; Result(a, CommandReturn(b)),\n    Command,\n  ) -&gt; c) -&gt; c\n Add a named option to the command. Bring your own parser.\n\n The continuation function gets a parser for the option and the command\n with the option added, for further configuration.\nSynonyms:\noutil/opt.with_named_option\noutil/opt with_named_option","url":"outil/opt.html#with_named_option"},{"doc":"outil/opt","title":"outil/opt","content":"","url":"outil/opt.html"}]);